var
	const
		TYPE_NONE   = 0
		TYPE_EXPIRE = 1
		TYPE_LIST   = 2
		TYPE_DEFAULT  = TYPE_EXPIRE
		DBQUERY_TIMEOUT = 600// 60s timeout on non-locked queries.
		DBQUERY_BUSY_INTERVAL = 20

DBQuery
	proc
		LockQuery()
		Used()

	var
		busy = 0
		expires
		locked = 0

	Used()
		expires = (mysql.mysql_type == TYPE_EXPIRE ? world.realtime + DBQUERY_TIMEOUT : world.realtime + DBQUERY_BUSY_INTERVAL)

	GetRowData()
		. = ..()
		Used()

	NextRow()
		. = ..()
		Used()

	Execute()
		. = ..()
		Used()

var/mysql_connection/mysql
mysql_connection
	var
		mysql_type = TYPE_DEFAULT
		default_queries_size = 15
		maximum_queries_size = 25
		DBConnection/dbcon
		list/queries

	proc
		DBI()
		DBAddr()
		DBUser()
		DBPass()
		DBName()
		CreateQuery()
		CleanQueries()
		SetupQueries()

	New()
		dbcon = new()

	CleanQueries()
		set background = 1
		spawn() while(1)
			if(queries) for(var/DBQuery/Q in queries)
				if(world.realtime >= Q.expires && !Q.locked)
					if(mysql_type == TYPE_EXPIRE)
						del Q
						if(!queries.len)
							queries = null
					else if(mysql_type == TYPE_LIST)
						Q.Close()
						Q.busy = 0
			sleep(mysql_type == TYPE_EXPIRE ? DBQUERY_TIMEOUT : DBQUERY_BUSY_INTERVAL)

	SetupQueries()
		queries = new()
		for(var/i = 1, i <= default_queries_size, i++)
			var/DBQuery/Q = dbcon.NewQuery()
			queries += Q

	CreateQuery()
		if(!dbcon.IsConnected())
			var/ok = dbcon.Connect(DBI(), DBUser(), DBPass())
			if(!ok)
				return "Unable to initialize database connection ([dbcon.ErrorMsg()])"
		var/DBQuery/Q

		if(mysql_type == TYPE_EXPIRE)
			Q = dbcon.NewQuery()
			if(!queries) queries = new()
			queries += Q
			return Q

		else if(mysql_type == TYPE_NONE)
			Q = dbcon.NewQuery()
			return Q

		else if(mysql_type == TYPE_LIST)
			if(!queries)
				SetupQueries()

			. = 0
			for(Q in queries)
				if(!Q.busy)
					. = 1
					Q.busy = 1
					break

			if(!.)
				if(length(queries) >= maximum_queries_size) // No more!
					return "Unable to create new query: Maximum query size list reached"

				Q = dbcon.NewQuery()
				queries += Q
			return Q

	DBUser()
		return "mysql_user_here"

	DBPass()
		return "mysql_pass_here"

	DBName()
		return "db_name_here"

	DBAddr()
		return "localhost"

	DBI()
		return "dbi:mysql:[DBName()]:[DBAddr()]:3306"